@@ -1,4 +1,4 @@
-# Copyright 2015, 2016 IBM Corp.
+# Copyright 2015, 2018 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -63,13 +63,14 @@ class BaseVSCSITest(test_vol.TestVolumeAdapter):
         @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
         def init_vol_adpt(mock_pvm_uuid, mock_getter):
             con_info = {
+                'serial': 'id',
                 'data': {
                     'initiator_target_map': {
                         p_wwpn1: ['t1'],
                         p_wwpn2: ['t2', 't3']
                     },
                     'target_lun': '1',
-                    'volume_id': 'id'
+                    'volume_id': 'a_volume_identifier',
                 },
             }
             mock_inst = mock.MagicMock()
@@ -210,6 +211,7 @@ class TestVSCSIAdapter(BaseVSCSITest):
             self.assertIsInstance(vios_w, pvm_vios.VIOS)
             self.assertEqual('1234', lpar_uuid)
             self.assertIsInstance(pv, pvm_stor.PV)
+            self.assertEqual('a_volume_identifier', pv.tag)
             self.assertEqual(62, lpar_slot_num)
             self.assertEqual('the_lua', lua)
             return 'fake_map'
@@ -290,6 +292,10 @@ class TestVSCSIAdapter(BaseVSCSITest):
         self.assertRaises(p_exc.VolumeAttachFailed,
                           self.vol_drv._validate_vios_on_connection, 1)
 
+    def test_extend_volume(self):
+        # Ensure the method is implemented
+        self.vol_drv.extend_volume()
+
     @mock.patch('pypowervm.tasks.hdisk.remove_hdisk')
     @mock.patch('pypowervm.wrappers.virtual_io_server.VIOS.hdisk_from_uuid')
     @mock.patch('pypowervm.tasks.scsi_mapper.remove_maps')
