@@ -1,4 +1,4 @@
-# Copyright 2014, 2017 IBM Corp.
+# Copyright IBM Corp. and contributors
 #
 # All Rights Reserved.
 #
@@ -66,7 +66,7 @@ from nova_powervm.virt.powervm.tasks import storage as tf_stg
 from nova_powervm.virt.powervm.tasks import vm as tf_vm
 from nova_powervm.virt.powervm import vm
 from nova_powervm.virt.powervm import volume as vol_attach
-
+from nova_powervm.virt.powervm.volume import iscsi
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
@@ -90,19 +90,18 @@ class PowerVMDriver(driver.ComputeDriver):
 
     """PowerVM Implementation of Compute Driver."""
 
-    capabilities = {
-        "has_imagecache": False,
-        "supports_recreate": True,
-        "supports_migrate_to_same_host": False,
-        "supports_attach_interface": True,
-        "supports_device_tagging": False,
-        "supports_tagged_attach_interface": False,
-        "supports_tagged_attach_volume": False,
-        "supports_extend_volume": True,
-        "supports_multiattach": False,
-    }
-
     def __init__(self, virtapi):
+        self.capabilities = {
+            # NOTE(edmondsw): 'has_imagecache' will be set dynamically
+            "supports_recreate": True,
+            "supports_migrate_to_same_host": False,
+            "supports_attach_interface": True,
+            "supports_device_tagging": False,
+            "supports_tagged_attach_interface": False,
+            "supports_tagged_attach_volume": False,
+            "supports_extend_volume": True,
+            "supports_multiattach": False,
+        }
         super(PowerVMDriver, self).__init__(virtapi)
 
     def init_host(self, host):
@@ -177,7 +176,8 @@ class PowerVMDriver(driver.ComputeDriver):
         self.disk_dvr = importutils.import_object_ns(
             DISK_ADPT_NS, DISK_ADPT_MAPPINGS[CONF.powervm.disk_driver.lower()],
             self.adapter, self.host_uuid)
-        self.capabilities.update(self.disk_dvr.capabilities)
+        has_imagecache = self.disk_dvr.capabilities['has_imagecache']
+        self.capabilities['has_imagecache'] = has_imagecache
 
     def manage_image_cache(self, context, all_instances):
         self.disk_dvr.manage_image_cache(context, all_instances)
@@ -392,8 +392,7 @@ class PowerVMDriver(driver.ComputeDriver):
         # Build the PowerVM Slot lookup map.  Only the recreate action needs
         # the volume driver iterator (to look up volumes and their client
         # mappings).
-        vol_drv_iter = (self._vol_drv_iter(context, instance, bdms,
-                                           stg_ftsk=stg_ftsk)
+        vol_drv_iter = (self._vol_drv_iter(instance, bdms, stg_ftsk=stg_ftsk)
                         if recreate else None)
         slot_mgr = slot.build_slot_mgr(
             instance, self.store_api, adapter=self.adapter,
@@ -493,7 +492,7 @@ class PowerVMDriver(driver.ComputeDriver):
         :param slot_mgr: A NovaSlotManager.  Used to store/retrieve the client
                          slots used when a volume is attached to a VM.
         """
-        for bdm, vol_drv in self._vol_drv_iter(context, instance, bdms,
+        for bdm, vol_drv in self._vol_drv_iter(instance, bdms,
                                                stg_ftsk=stg_ftsk):
             # First connect the volume.  This will update the
             # connection_info.
@@ -519,7 +518,7 @@ class PowerVMDriver(driver.ComputeDriver):
                          slots used when a volume is detached from a VM.
         """
         # TODO(thorst) Do we need to do something on the disconnect for slots?
-        for bdm, vol_drv in self._vol_drv_iter(context, instance, bdms,
+        for bdm, vol_drv in self._vol_drv_iter(instance, bdms,
                                                stg_ftsk=stg_ftsk):
             flow.add(tf_stg.DisconnectVolume(vol_drv, slot_mgr))
 
@@ -755,7 +754,13 @@ class PowerVMDriver(driver.ComputeDriver):
         instance.save()
 
     def extend_volume(self, connection_info, instance):
-        """Resize an attached volume"""
+        """Extend the disk attached to the instance.
+
+        :param dict connection_info: The connection for the extended volume.
+        :param nova.objects.instance.Instance instance:
+            The instance whose volume gets extended.
+        :return: None
+        """
         vol_drv = vol_attach.build_volume_driver(
             self.adapter, self.host_uuid, instance, connection_info)
         vol_drv.extend_volume()
@@ -765,6 +770,9 @@ class PowerVMDriver(driver.ComputeDriver):
         """Detach the volume attached to the instance."""
         self._log_operation('detach_volume', instance)
 
+        # Define the flow
+        flow = tf_lf.Flow("detach_volume")
+
         # Get a volume adapter for this volume
         vol_drv = vol_attach.build_volume_driver(
             self.adapter, self.host_uuid, instance, connection_info)
@@ -784,9 +792,6 @@ class PowerVMDriver(driver.ComputeDriver):
                 mig.cleanup_volume(vol_drv)
             return
 
-        # Define the flow
-        flow = tf_lf.Flow("detach_volume")
-
         # Add a task to detach the volume
         slot_mgr = slot.build_slot_mgr(instance, self.store_api)
         flow.add(tf_stg.DisconnectVolume(vol_drv, slot_mgr))
@@ -801,26 +806,21 @@ class PowerVMDriver(driver.ComputeDriver):
         """Snapshots the specified instance.
 
         :param context: security context
-        :param instance: Instance object as returned by DB layer.
-        :param image_id: Reference to a pre-created image that will
-                         hold the snapshot.
-        :param update_task_state: Callable to update the state of the snapshot
-                                  task with one of the IMAGE_* consts from
-                                  nova.compute.task_states.  Call spec
-                                  (inferred from compute driver source):
-            update_task_state(task_state, expected_task_state=None)
-                param task_state: The nova.compute.task_states.IMAGE_* state to
-                                  set.
-                param expected_state: The nova.compute.task_state.IMAGE_* state
-                                      which should be in place before this
-                                      update.  The driver will raise if this
-                                      doesn't match.
+        :param instance: nova.objects.instance.Instance
+        :param image_id: Reference to a pre-created image that will hold the
+                         snapshot.
+        :param update_task_state: Callback function to update the task_state
+            on the instance while the snapshot operation progresses. The
+            function takes a task_state argument and an optional
+            expected_task_state kwarg which defaults to
+            nova.compute.task_states.IMAGE_SNAPSHOT. See
+            nova.objects.instance.Instance.save for expected_task_state usage.
         """
         if not self.disk_dvr.capabilities.get('snapshot'):
             raise exception.NotSupportedWithOption(
                 message=_("The snapshot operation is not supported in "
-                          "conjunction with a CONF.powervm.disk_driver "
-                          "setting of %s.") % CONF.powervm.disk_driver)
+                          "conjunction with a [powervm]/disk_driver setting "
+                          "of %s.") % CONF.powervm.disk_driver)
         self._log_operation('snapshot', instance)
 
         # Define the flow
@@ -1103,7 +1103,9 @@ class PowerVMDriver(driver.ComputeDriver):
             connector["wwpns"] = wwpn_list
         connector["multipath"] = CONF.powervm.volume_use_multipath
         connector['host'] = vol_attach.get_hostname_for_volume(instance)
-        connector['initiator'] = vol_attach.get_iscsi_initiator(self.adapter)
+        initiator_dict = iscsi.get_iscsi_initiators(self.adapter)
+        if initiator_dict:
+            connector['initiator'] = list(initiator_dict.values())[0]
         return connector
 
     def migrate_disk_and_power_off(self, context, instance, dest,
@@ -1164,8 +1166,8 @@ class PowerVMDriver(driver.ComputeDriver):
             # adapters in the same slots.
             slot_mgr = slot.build_slot_mgr(
                 instance, self.store_api, adapter=self.adapter,
-                vol_drv_iter=self._vol_drv_iter(
-                    context, instance, bdms, stg_ftsk=stg_ftsk))
+                vol_drv_iter=self._vol_drv_iter(instance, bdms,
+                                                stg_ftsk=stg_ftsk))
 
             # Determine if there are volumes to disconnect.  If so, remove each
             # volume (within the transaction manager)
@@ -1261,8 +1263,8 @@ class PowerVMDriver(driver.ComputeDriver):
             # b) If adding/removing block devices, to register the slots.
             slot_mgr = slot.build_slot_mgr(
                 instance, self.store_api, adapter=self.adapter,
-                vol_drv_iter=self._vol_drv_iter(
-                    context, instance, bdms, stg_ftsk=stg_ftsk))
+                vol_drv_iter=self._vol_drv_iter(instance, bdms,
+                                                stg_ftsk=stg_ftsk))
         else:
             stg_ftsk = None
 
@@ -1646,40 +1648,35 @@ class PowerVMDriver(driver.ComputeDriver):
         :param instance: instance object that is migrated
         :param network_info: instance network information
         :param block_migration: if true, post operation of block_migration.
+        :param block_device_info: instance block device information.
         """
         LOG.info("Post live migration processing on destination host.",
                  instance=instance)
         mig = self.live_migrations[instance.uuid]
         mig.instance = instance
 
-        # Build the volume drivers
-        vol_drvs = self._build_vol_drivers(context, instance,
-                                           block_device_info)
+        bdms = self._extract_bdm(block_device_info)
+        # Get a volume driver iterator for volume and BDM mapping
+        vol_drv_iter = self._vol_drv_iter(instance, bdms)
 
         # Run post live migration
-        mig.post_live_migration_at_destination(network_info, vol_drvs)
+        mig.post_live_migration_at_destination(network_info, vol_drv_iter)
         del self.live_migrations[instance.uuid]
 
-    def _vol_drv_iter(self, context, instance, bdms, stg_ftsk=None):
+    def _vol_drv_iter(self, instance, bdms, stg_ftsk=None):
         """Yields a bdm and volume driver."""
         # Get a volume driver for each volume
         for bdm in bdms or []:
-            conn_info = bdm.get('connection_info')
-            # if it doesn't have connection_info, it's not a volume
-            if not conn_info:
-                continue
-
             vol_drv = vol_attach.build_volume_driver(
-                self.adapter, self.host_uuid, instance, conn_info,
-                stg_ftsk=stg_ftsk)
+                self.adapter, self.host_uuid, instance,
+                bdm.get('connection_info'), stg_ftsk=stg_ftsk)
             yield bdm, vol_drv
 
     def _build_vol_drivers(self, context, instance, block_device_info):
         """Builds the volume connector drivers for a block device info."""
         # Get a volume driver for each volume
         bdms = self._extract_bdm(block_device_info)
-        return [vol_drv for bdm, vol_drv in self._vol_drv_iter(
-            context, instance, bdms)]
+        return [vol_drv for bdm, vol_drv in self._vol_drv_iter(instance, bdms)]
 
     def unfilter_instance(self, instance, network_info):
         """Stop filtering instance."""
@@ -1694,7 +1691,7 @@ class PowerVMDriver(driver.ComputeDriver):
         classes from nova.virt.block_device.  Each block device
         represents one volume connection.
 
-        An example string representation of the a DriverVolumeBlockDevice
+        An example string representation of a DriverVolumeBlockDevice
         from the early Liberty time frame is:
         {'guest_format': None,
         'boot_index': 0,
