@@ -22,6 +22,7 @@ from nova import exception
 from nova.objects import migrate_data as mig_obj
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
+from oslo_utils import excutils
 from pypowervm.tasks import management_console as mgmt_task
 from pypowervm.tasks import migration as mig
 from pypowervm.tasks import storage as stor_task
@@ -30,12 +31,12 @@ from pypowervm import util
 
 from nova_powervm import conf as cfg
 from nova_powervm.virt.powervm.i18n import _
-from nova_powervm.virt.powervm.i18n import _LE
-from nova_powervm.virt.powervm.i18n import _LI
 from nova_powervm.virt.powervm import media
+from nova_powervm.virt.powervm.tasks import storage as tf_stg
 from nova_powervm.virt.powervm import vif
 from nova_powervm.virt.powervm import vm
 
+
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
@@ -143,9 +144,8 @@ class LiveMigrationDest(LiveMigration):
         :param migrate_data: a PowerVMLiveMigrateData object
         :param vol_drvs: volume drivers for the attached volumes
         """
-        LOG.debug('Running pre live migration on destination.',
-                  instance=self.instance)
-        LOG.debug('Migration data: %s', migrate_data, instance=self.instance)
+        LOG.debug('Running pre live migration on destination. Migration data: '
+                  '%s', migrate_data, instance=self.instance)
 
         # Set the ssh auth key.
         mgmt_task.add_authorized_key(self.drvr.adapter,
@@ -162,13 +162,14 @@ class LiveMigrationDest(LiveMigration):
 
         # For each volume, make sure it's ready to migrate
         for vol_drv in vol_drvs:
-            LOG.info(_LI('Performing pre migration for volume %(volume)s'),
+            LOG.info('Performing pre migration for volume %(volume)s',
                      dict(volume=vol_drv.volume_id), instance=self.instance)
             try:
                 vol_drv.pre_live_migration_on_destination(
                     migrate_data.vol_data)
-            except Exception as e:
-                LOG.exception(e, instance=self.instance)
+            except Exception:
+                LOG.exception("PowerVM error preparing instance for live "
+                              "migration.", instance=self.instance)
                 # It failed.
                 vol_exc = LiveMigrationVolume(
                     host=self.drvr.host_wrapper.system_name,
@@ -183,28 +184,29 @@ class LiveMigrationDest(LiveMigration):
         self.pre_live_vol_data = migrate_data.vol_data
         return migrate_data
 
-    def post_live_migration_at_destination(self, network_infos, vol_drvs):
+    def post_live_migration_at_destination(self, network_infos, vol_drv_iter):
         """Do post migration cleanup on destination host.
 
         :param network_infos: instance network information
-        :param vol_drvs: volume drivers for the attached volumes
+        :param vol_drv_iter: volume driver iterator for the attached volumes
+                             and BDM information.
         """
         # The LPAR should be on this host now.
         LOG.debug("Post live migration at destination.",
                   instance=self.instance)
 
-        # An unbounded dictionary that each volume adapter can use to persist
-        # data from one call to the next.
-        mig_vol_stor = {}
-
         # For each volume, make sure it completes the migration
-        for vol_drv in vol_drvs:
-            LOG.info(_LI('Performing post migration for volume %(volume)s'),
+        for bdm, vol_drv in vol_drv_iter:
+            LOG.info('Performing post migration for volume %(volume)s',
                      dict(volume=vol_drv.volume_id), instance=self.instance)
             try:
-                vol_drv.post_live_migration_at_destination(mig_vol_stor)
-            except Exception as e:
-                LOG.exception(e, instance=self.instance)
+                vol_drv.post_live_migration_at_destination(
+                    self.pre_live_vol_data)
+                # Save the BDM for the updated connection info.
+                tf_stg.SaveBDM(bdm, self.instance).execute()
+            except Exception:
+                LOG.exception("PowerVM error cleaning up destination host "
+                              "after migration.", instance=self.instance)
                 # It failed.
                 raise LiveMigrationVolume(
                     host=self.drvr.host_wrapper.system_name,
@@ -235,14 +237,15 @@ class LiveMigrationDest(LiveMigration):
 
         :param vol_drv: volume driver for the attached volume
         """
-        LOG.info(_LI('Performing detach for volume %(volume)s'),
+        LOG.info('Performing detach for volume %(volume)s',
                  dict(volume=vol_drv.volume_id), instance=self.instance)
         # Ensure the volume data is present before trying cleanup
         if hasattr(self, 'pre_live_vol_data'):
             try:
                 vol_drv.cleanup_volume_at_destination(self.pre_live_vol_data)
-            except Exception as e:
-                LOG.exception(e, instance=self.instance)
+            except Exception:
+                LOG.exception("PowerVM error cleaning volume after failed "
+                              "migration.", instance=self.instance)
                 # Log the exception but no need to raise one because
                 # the VM is still on the source host.
 
@@ -268,7 +271,8 @@ class LiveMigrationSrc(LiveMigration):
         lpar_w = vm.get_instance_wrapper(self.drvr.adapter, self.instance)
         self.lpar_w = lpar_w
 
-        LOG.debug('Dest Migration data: %s', self.mig_data)
+        LOG.debug('Dest Migration data: %s', self.mig_data,
+                  instance=self.instance)
 
         # Check proc compatibility modes
         if (lpar_w.proc_compat_mode and lpar_w.proc_compat_mode not in
@@ -305,7 +309,7 @@ class LiveMigrationSrc(LiveMigration):
             vol_drv.pre_live_migration_on_source(vol_data)
         self.mig_data.vol_data = vol_data
 
-        LOG.debug('Src Migration data: %s', self.mig_data,
+        LOG.debug('Source migration data: %s', self.mig_data,
                   instance=self.instance)
 
         # Create a FeedTask to scrub any orphaned mappings/storage associated
@@ -314,9 +318,8 @@ class LiveMigrationSrc(LiveMigration):
         stg_ftsk = stor_task.ScrubOrphanStorageForLpar(self.drvr.adapter,
                                                        lpar_w.id)
         # Add subtasks to remove the VOpt devices under the same FeedTask.
-        media.ConfigDrivePowerVM(self.drvr.adapter, self.drvr.host_uuid
-                                 ).dlt_vopt(lpar_w.uuid, stg_ftsk=stg_ftsk,
-                                            remove_mappings=False)
+        media.ConfigDrivePowerVM(self.drvr.adapter).dlt_vopt(
+            lpar_w.uuid, stg_ftsk=stg_ftsk, remove_mappings=False)
         # Now execute the FeedTask, performing both scrub and VOpt removal.
         stg_ftsk.execute()
 
@@ -331,8 +334,8 @@ class LiveMigrationSrc(LiveMigration):
         :param context: security context
         :param migrate_data: a PowerVMLiveMigrateData object
         """
-        LOG.debug("Starting migration.", instance=self.instance)
-        LOG.debug("Migrate data: %s", migrate_data, instance=self.instance)
+        LOG.debug("Starting migration. Migrate data: %s", migrate_data,
+                  instance=self.instance)
 
         # The passed in mig data has more info (dest data added), so replace
         self.mig_data = migrate_data
@@ -373,8 +376,8 @@ class LiveMigrationSrc(LiveMigration):
             for trunk_to_del in trunks_to_del:
                 trunk_to_del.delete()
         except Exception:
-            LOG.error(_LE("Live migration failed."), instance=self.instance)
-            raise
+            with excutils.save_and_reraise_exception(logger=LOG):
+                LOG.exception("Live migration failed.", instance=self.instance)
         finally:
             LOG.debug("Finished migration.", instance=self.instance)
 
@@ -397,12 +400,13 @@ class LiveMigrationSrc(LiveMigration):
         """
         # For each volume, make sure the source is cleaned
         for vol_drv in vol_drvs:
-            LOG.info(_LI('Performing post migration for volume %(volume)s'),
+            LOG.info('Performing post migration for volume %(volume)s',
                      dict(volume=vol_drv.volume_id), instance=self.instance)
             try:
                 vol_drv.post_live_migration_at_source(migrate_data.vol_data)
-            except Exception as e:
-                LOG.exception(e, instance=self.instance)
+            except Exception:
+                LOG.exception("PowerVM error cleaning source host after live "
+                              "migration.", instance=self.instance)
                 # Log the exception but no need to raise one because
                 # the VM is already moved.  By raising an exception that
                 # results in the VM being on the new host but the instance
@@ -435,9 +439,8 @@ class LiveMigrationSrc(LiveMigration):
             if self.lpar_w.migration_state != 'Not_Migrating':
                 self.migration_recover()
 
-        except Exception as ex:
-            LOG.error(_LE("Migration recover failed with error: %s"), ex,
-                      instance=self.instance)
+        except Exception:
+            LOG.exception("Migration rollback failed.", instance=self.instance)
         finally:
             LOG.debug("Finished migration rollback.", instance=self.instance)
 
@@ -463,11 +466,9 @@ class LiveMigrationSrc(LiveMigration):
         LOG.debug("Abort migration.", instance=self.instance)
         try:
             mig.migrate_abort(self.lpar_w)
-        except Exception as ex:
-            LOG.error(_LE("Abort of live migration has failed. "
-                          "This is non-blocking. Exception is logged below."),
-                      instance=self.instance)
-            LOG.exception(ex, instance=self.instance)
+        except Exception:
+            LOG.exception("Abort of live migration has failed. This is "
+                          "non-blocking.", instance=self.instance)
 
     def migration_recover(self):
         """Recover migration if the migration failed for any reason. """
