@@ -1,4 +1,4 @@
-# Copyright 2016, 2017 IBM Corp.
+# Copyright 2016, 2018 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -20,14 +20,13 @@ import six
 from nova import context as ctx
 from nova import exception
 from nova import network as net_api
-from nova.network import linux_net
 from nova.network import model as network_model
-from nova import utils
 from oslo_config import cfg
 from oslo_log import log
 from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import importutils
+import pypowervm.const as pvm_c
 from pypowervm import exceptions as pvm_ex
 from pypowervm.tasks import cna as pvm_cna
 from pypowervm.tasks import partition as pvm_par
@@ -53,18 +52,22 @@ EVENT_PROVIDER_ID = 'NOVA_PVM_VIF'
 
 VIF_TYPE_PVM_SEA = 'pvm_sea'
 VIF_TYPE_PVM_OVS = 'ovs'
-VIF_TYPE_PVM_BRIDGE = 'bridge'
 VIF_TYPE_PVM_SRIOV = 'pvm_sriov'
 
 VIF_MAPPING = {VIF_TYPE_PVM_SEA:
                'nova_powervm.virt.powervm.vif.PvmSeaVifDriver',
                VIF_TYPE_PVM_OVS:
                'nova_powervm.virt.powervm.vif.PvmOvsVifDriver',
-               VIF_TYPE_PVM_BRIDGE:
-               'nova_powervm.virt.powervm.vif.PvmLBVifDriver',
                VIF_TYPE_PVM_SRIOV:
                'nova_powervm.virt.powervm.vif.PvmVnicSriovVifDriver'}
 
+# NOTE(svenkat): Manually adjust CNA child ordering to workaround bug 1731657
+# TODO(svenkat) Remove workaround when pypowervm is fixed
+child_order = list(pvm_net.CNA._child_order)
+child_order.remove('VirtualNetworks')
+child_order.append('VirtualNetworks')
+pvm_net.CNA._child_order = tuple(child_order)
+
 CONF = cfg.CONF
 
 
@@ -316,6 +319,21 @@ def plug_secure_rmc_vif(adapter, instance, host_uuid, slot_mgr):
     return cna_w
 
 
+def _get_trunk_dev_name(vif):
+    """Returns the device name for the trunk adapter.
+
+    A given VIF will have a trunk adapter and a
+    client adapter.  This will return the trunk adapter's name as it
+    will appear on the management VM.
+
+    :param vif: The nova network interface
+    :return: The device name.
+    """
+    if 'devname' in vif:
+        return vif['devname']
+    return ("nic" + vif['id'])[:network_model.NIC_NAME_LEN]
+
+
 @six.add_metaclass(abc.ABCMeta)
 class PvmVifDriver(object):
     """Represents an abstract class for a PowerVM Vif Driver.
@@ -487,139 +505,6 @@ class PvmSeaVifDriver(PvmVifDriver):
         return cna_w
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PvmLioVifDriver(PvmVifDriver):
-    """An abstract VIF driver that uses Linux I/O to host."""
-
-    @staticmethod
-    def get_trunk_dev_name(vif):
-        """Returns the device name for the trunk adapter.
-
-        A given VIF in the Linux I/O model will have a trunk adapter and a
-        client adapter.  This will return the trunk adapter's name as it
-        will appear on the management VM.
-
-        :param vif: The nova network interface
-        :return: The device name.
-        """
-        if 'devname' in vif:
-            return vif['devname']
-        return ("nic" + vif['id'])[:network_model.NIC_NAME_LEN]
-
-    def plug(self, vif, slot_num, new_vif=True):
-        """Plugs a virtual interface (network) into a VM.
-
-        Creates a 'peer to peer' connection between the Management partition
-        hosting the Linux I/O and the client VM.  There will be one trunk
-        adapter for a given client adapter.
-
-        The device will be 'up' on the mgmt partition.
-
-        :param vif: The virtual interface to plug into the instance.
-        :param slot_num: Which slot number to plug the VIF into.  May be None.
-        :param new_vif: (Optional, Default: True) If set, indicates that it is
-                        a brand new VIF.  If False, it indicates that the VIF
-                        is already on the client but should be treated on the
-                        bridge.
-        :return: The new vif that was created.  Only returned if new_vif is
-                 set to True.  Otherwise None is expected.
-        """
-        dev_name = self.get_trunk_dev_name(vif)
-
-        if new_vif:
-            # Create the trunk and client adapter.
-            lpar_uuid = vm.get_pvm_uuid(self.instance)
-            mgmt_uuid = pvm_par.get_this_partition(self.adapter).uuid
-            cna_w = pvm_cna.crt_p2p_cna(
-                self.adapter, self.host_uuid, lpar_uuid, [mgmt_uuid],
-                CONF.powervm.pvm_vswitch_for_novalink_io, crt_vswitch=True,
-                mac_addr=vif['address'], dev_name=dev_name,
-                slot_num=slot_num)[0]
-        else:
-            cna_w = None
-
-        # Make sure to just run the up just in case.
-        utils.execute('ip', 'link', 'set', dev_name, 'up', run_as_root=True)
-
-        return cna_w
-
-
-class PvmLBVifDriver(PvmLioVifDriver):
-    """The Linux Bridge VIF driver for PowerVM."""
-
-    def plug(self, vif, slot_num, new_vif=True):
-        """Plugs a virtual interface (network) into a VM.
-
-        Extends the base Lio implementation.  Will make sure that the bridge
-        supports the trunk adapter.
-
-        :param vif: The virtual interface to plug into the instance.
-        :param slot_num: Which slot number to plug the VIF into.  May be None.
-        :param new_vif: (Optional, Default: True) If set, indicates that it is
-                        a brand new VIF.  If False, it indicates that the VIF
-                        is already on the client but should be treated on the
-                        bridge.
-        :return: The new vif that was created.  Only returned if new_vif is
-                 set to True.  Otherwise None is expected.
-        """
-        # Only call the parent if it is truly a new VIF
-        if new_vif:
-            cna_w = super(PvmLBVifDriver, self).plug(vif, slot_num)
-        else:
-            cna_w = None
-
-        # Similar to libvirt's vif.py plug_bridge.  Need to attach the
-        # interface to the bridge.
-        linux_net.LinuxBridgeInterfaceDriver.ensure_bridge(
-            vif['network']['bridge'], self.get_trunk_dev_name(vif))
-
-        return cna_w
-
-    def unplug(self, vif, cna_w_list=None):
-        """Unplugs a virtual interface (network) from a VM.
-
-        Extends the base implementation, but before invoking it will remove
-        itself from the bridge it is connected to and delete the corresponding
-        trunk device on the mgmt partition.
-
-        :param vif: The virtual interface to plug into the instance.
-        :param cna_w_list: (Optional, Default: None) The list of Client Network
-                           Adapters from pypowervm.  Providing this input
-                           allows for an improvement in operation speed.
-        :return cna_w: The deleted Client Network Adapter.
-        """
-        # Need to find the adapters if they were not provided
-        if not cna_w_list:
-            cna_w_list = vm.get_cnas(self.adapter, self.instance)
-
-        # Find the CNA for this vif.
-        cna_w = self._find_cna_for_vif(cna_w_list, vif)
-        if not cna_w:
-            LOG.warning('Unable to unplug VIF with mac %(mac)s.  The VIF was '
-                        'not found on the instance.',
-                        {'mac': vif['address']}, instance=self.instance)
-            return None
-
-        # Find and delete the trunk adapters
-        trunks = pvm_cna.find_trunks(self.adapter, cna_w)
-
-        dev_name = self.get_trunk_dev_name(vif)
-        utils.execute('ip', 'link', 'set', dev_name, 'down', run_as_root=True)
-        try:
-            utils.execute('brctl', 'delif', vif['network']['bridge'],
-                          dev_name, run_as_root=True)
-        except Exception:
-            LOG.exception(
-                'Unable to delete device %(dev_name)s from bridge %(bridge)s.',
-                {'dev_name': dev_name, 'bridge': vif['network']['bridge']},
-                instance=self.instance)
-        for trunk in trunks:
-            trunk.delete()
-
-        # Now delete the client CNA
-        return super(PvmLBVifDriver, self).unplug(vif, cna_w_list=cna_w_list)
-
-
 class PvmVnicSriovVifDriver(PvmVifDriver):
     """The SR-IOV VIF driver for PowerVM."""
 
@@ -669,15 +554,25 @@ class PvmVnicSriovVifDriver(PvmVifDriver):
 
         # Capacity: plugin sets from binding:profile, then conf, then default
         capacity = vif['details']['capacity']
+        maxcapacity = vif['details'].get('maxcapacity')
 
         vnic = pvm_card.VNIC.bld(
             self.adapter, vlan_id, slot_num=slot_num, mac_addr=mac_address,
             allowed_vlans=pvm_util.VLANList.NONE,
             allowed_macs=pvm_util.MACList.NONE)
 
-        sriovtask.set_vnic_back_devs(vnic, pports, sys_w=msys,
-                                     redundancy=redundancy,
-                                     capacity=capacity, check_port_status=True)
+        try:
+            sriovtask.set_vnic_back_devs(vnic, pports, sys_w=msys,
+                                         redundancy=redundancy,
+                                         capacity=capacity,
+                                         max_capacity=maxcapacity,
+                                         check_port_status=True)
+        except ValueError as ve:
+            LOG.exception("Failed to set vNIC backing devices")
+            msg = ''
+            if ve.args:
+                msg = ve.args[0]
+            raise exception.VirtualInterfacePlugException(message=msg)
 
         return vnic.create(parent_type=pvm_lpar.LPAR,
                            parent_uuid=vm.get_pvm_uuid(self.instance))
@@ -695,7 +590,28 @@ class PvmVnicSriovVifDriver(PvmVifDriver):
         return vnic
 
 
-class PvmOvsVifDriver(PvmLioVifDriver):
+class PvmMetaAttrs(list):
+    """Represents meta attributes for a PowerVM Vif Driver.
+
+    """
+
+    def __init__(self, vif, instance):
+        """Initializes meta attributes.
+
+        :param vif: The virtual interface for the instance
+        :param instance: The nova instance that the vif action will be run
+                         against.
+        """
+        self.append('iface-id=%s' % (vif.get('ovs_interfaceid') or vif['id']))
+        self.append('iface-status=active')
+        self.append('attached-mac=%s' % vif['address'])
+        self.append('vm-uuid=%s' % instance.uuid)
+
+    def __str__(self):
+        return ','.join(self)
+
+
+class PvmOvsVifDriver(PvmVifDriver):
     """The Open vSwitch VIF driver for PowerVM."""
 
     def plug(self, vif, slot_num, new_vif=True):
@@ -714,18 +630,53 @@ class PvmOvsVifDriver(PvmLioVifDriver):
         :return: The new vif that was created.  Only returned if new_vif is
                  set to True.  Otherwise None is expected.
         """
-        cna_w = super(PvmOvsVifDriver, self).plug(vif, slot_num,
-                                                  new_vif=new_vif)
-        dev_name = self.get_trunk_dev_name(vif)
-        # There will only be one trunk wrap, as we have created with just the
-        # mgmt lpar.  Next step is to connect to the OVS.
-        mtu = vif['network'].get_meta('mtu')
-        linux_net.create_ovs_vif_port(vif['network']['bridge'], dev_name,
-                                      self.get_ovs_interfaceid(vif),
-                                      vif['address'], self.instance.uuid,
-                                      mtu=mtu)
+        lpar_uuid = vm.get_pvm_uuid(self.instance)
+        mgmt_uuid = pvm_par.get_mgmt_partition(self.adapter).uuid
 
-        return cna_w
+        # There will only be one trunk wrap, as we have created with just
+        # the mgmt lpar.  Next step is to connect to the OVS.
+        mtu = vif['network'].get_meta('mtu')
+        dev_name = _get_trunk_dev_name(vif)
+
+        meta_attrs = PvmMetaAttrs(vif, self.instance)
+
+        if new_vif:
+            # Create the trunk and client adapter.
+            return pvm_cna.crt_p2p_cna(
+                self.adapter, self.host_uuid, lpar_uuid, [mgmt_uuid],
+                CONF.powervm.pvm_vswitch_for_novalink_io, crt_vswitch=True,
+                mac_addr=vif['address'], dev_name=dev_name,
+                slot_num=slot_num, ovs_bridge=vif['network']['bridge'],
+                ovs_ext_ids=str(meta_attrs), configured_mtu=mtu)[0]
+        else:
+            # Bug : https://bugs.launchpad.net/nova-powervm/+bug/1731548
+            # When a host is rebooted, something is discarding tap devices for
+            # VMs deployed with OVS vif. To prevent VMs losing network
+            # connectivity, this is fixed by recreating the tap devices during
+            # init of the nova compute service, which will call vif plug with
+            # new_vif==False.
+
+            # Find the CNA for this vif.
+            # TODO(svenkat) improve performance by caching VIOS wrapper(s) and
+            # CNA lists (in case >1 vif per VM).
+            cna_w_list = vm.get_cnas(self.adapter, self.instance)
+            cna_w = self._find_cna_for_vif(cna_w_list, vif)
+            # Find the corresponding trunk adapter
+            trunks = pvm_cna.find_trunks(self.adapter, cna_w)
+            for trunk in trunks:
+                # Set MTU, OVS external ids, and OVS bridge metadata
+                # TODO(svenkat) set_parm_value calls should be replaced once
+                # pypowervm supports setting these values directly.
+                trunk.set_parm_value('ConfiguredMTU',
+                                     mtu, attrib=pvm_c.ATTR_KSV160)
+                trunk.set_parm_value('OvsPortExternalIds',
+                                     meta_attrs, attrib=pvm_c.ATTR_KSV160)
+                trunk.set_parm_value('OvsBridge',
+                                     vif['network']['bridge'],
+                                     attrib=pvm_c.ATTR_KSV160)
+                # Updating the trunk adapter will cause NovaLink to reassociate
+                # the tap device.
+                trunk.update()
 
     @staticmethod
     def get_ovs_interfaceid(vif):
@@ -765,8 +716,6 @@ class PvmOvsVifDriver(PvmLioVifDriver):
 
         # Find and delete the trunk adapters
         trunks = pvm_cna.find_trunks(self.adapter, cna_w)
-        dev = self.get_trunk_dev_name(vif)
-        linux_net.delete_ovs_vif_port(vif['network']['bridge'], dev)
         for trunk in trunks:
             trunk.delete()
 
@@ -790,21 +739,20 @@ class PvmOvsVifDriver(PvmLioVifDriver):
         """
         self._cleanup_orphan_adapters(vif,
                                       CONF.powervm.pvm_vswitch_for_novalink_io)
-        mgmt_wrap = pvm_par.get_this_partition(self.adapter)
-        dev = self.get_trunk_dev_name(vif)
+        mgmt_wrap = pvm_par.get_mgmt_partition(self.adapter)
+        dev = _get_trunk_dev_name(vif)
+
+        meta_attrs = PvmMetaAttrs(vif, self.instance)
+
+        mtu = vif['network'].get_meta('mtu')
 
         # Find a specific free VLAN and create the Trunk in a single atomic
         # action.
         cna_w = pvm_cna.crt_trunk_with_free_vlan(
             self.adapter, self.host_uuid, [mgmt_wrap.uuid],
-            CONF.powervm.pvm_vswitch_for_novalink_io, dev_name=dev)[0]
-
-        # Bring the vif up.  This signals to neutron that its ready for vif
-        # plugging
-        utils.execute('ip', 'link', 'set', dev, 'up', run_as_root=True)
-        linux_net.create_ovs_vif_port(vif['network']['bridge'], dev,
-                                      self.get_ovs_interfaceid(vif),
-                                      vif['address'], self.instance.uuid)
+            CONF.powervm.pvm_vswitch_for_novalink_io, dev_name=dev,
+            ovs_bridge=vif['network']['bridge'],
+            ovs_ext_ids=str(meta_attrs), configured_mtu=mtu)[0]
 
         # Save this data for the migration command.
         vea_vlan_mappings[vif['address']] = cna_w.pvid
@@ -838,12 +786,8 @@ class PvmOvsVifDriver(PvmLioVifDriver):
             self.adapter, parent_type=pvm_ms.System, one_result=True,
             name=CONF.powervm.pvm_vswitch_for_novalink_io).switch_id
 
-        # Delete port from OVS
-        linux_net.delete_ovs_vif_port(vif['network']['bridge'],
-                                      self.get_trunk_dev_name(vif))
-
         # Find the trunk
-        mgmt_wrap = pvm_par.get_this_partition(self.adapter)
+        mgmt_wrap = pvm_par.get_mgmt_partition(self.adapter)
         child_adpts = pvm_net.CNA.get(self.adapter, parent=mgmt_wrap)
         trunk = None
         for adpt in child_adpts:
@@ -889,8 +833,8 @@ class PvmOvsVifDriver(PvmLioVifDriver):
         :param vif: The virtual interface of an instance.  This may be
                     called network_info in other portions of the code.
         """
-        linux_net.delete_ovs_vif_port(vif['network']['bridge'],
-                                      self.get_trunk_dev_name(vif))
+        self._cleanup_orphan_adapters(vif,
+                                      CONF.powervm.pvm_vswitch_for_novalink_io)
 
     def _cleanup_orphan_adapters(self, vif, vswitch_name):
         """Finds and removes trunk VEAs that have no corresponding CNA."""
@@ -898,6 +842,4 @@ class PvmOvsVifDriver(PvmLioVifDriver):
         orphans = pvm_cna.find_orphaned_trunks(self.adapter, vswitch_name)
         for orphan in orphans:
             if vm.norm_mac(orphan.mac) == vif['address']:
-                linux_net.delete_ovs_vif_port(vif['network']['bridge'],
-                                              self.get_trunk_dev_name(vif))
                 orphan.delete()
